#include "myarea.h"
#include <cairomm/context.h>
#include <glibmm/main.h>

MyArea::MyArea()
{
	time=0;
	down = true;
	Glib::signal_timeout().connect( sigc::mem_fun(*this, &MyArea::on_timeout), resolution );
	init_objects();
}

MyArea::~MyArea()
{
}

#include<stdio.h>

void MyArea::bounce(){
	for(auto o : objects){
		o->yv = - ((random()%1000) / 100.0);
	}	
}

bool overlap(AnimObject *a, AnimObject *b){
	float sqdist = pow((a->x - b->x), 2) + pow((a->y - b->y), 2);
	return (sqdist < pow(a->r + b->r, 2));
}

bool MyArea::on_draw(const Cairo::RefPtr<Cairo::Context>& cr)
{
	Gtk::Allocation allocation = get_allocation();
	const int width = allocation.get_width();
	const int height = allocation.get_height();

	update_all(width, height);
	draw_all(cr);

	for(auto o : objects)
		for(auto c : objects)
			if(o != c && overlap(o, c))
				o->collide(c);
	cr->stroke();
	return true;
}

bool MyArea::on_timeout(){

	// force our program to redraw the entire clock.
	auto win = get_window();
	if (win)
	{
		Gdk::Rectangle r(0, 0, get_allocation().get_width(),
				get_allocation().get_height());
		win->invalidate_rect(r, false);
	}
	return true;
}

void MyArea::init_objects(){
	objects.push_back(new Ball(100, 500, 30, 1.0, 0));
	objects.push_back(new Ball(400, 550, 30, -1.0, 0));
	for(int i = 0; i < 5; i++)
		objects.push_back(new Ball(random() % 500, random() % 500, 30, (random()  % 200) / 200.0, 0));
// You'll want to add the hill to the objects list, probably something like this:
//	objects.push_back(new Hill(500,1000,200));
}

void MyArea::update_all(int width, int height){
	for(auto o : objects)
		o->update(resolution, width, height);
}

void MyArea::draw_all(const Cairo::RefPtr<Cairo::Context>& cr){
	for(auto o : objects)
		o->draw(cr);
}

Ball::Ball(int ix, int iy, int ir) : AnimObject(ix, iy, ir) {
	xv = yv = 0;
	lx = ix;
	ly = iy;
	mover = true;
}

Ball::Ball(int ix, int iy, int ir, float ixv, float iyv) : AnimObject(ix, iy, ir) {
	xv = ixv;
	yv = iyv;
	mover = true;
}

void Ball::back_up(){
	x = lx;
	y = ly;
}

void Ball::update(float time, int width, int height){
	lx = x;
	ly = y;
	float ag = GRAVITY * time * 5;
	yv += ag;
	x += xv * time * 5;
	y += yv * time * 5;
	if(x < 0){
		x = 0;
		xv = -xv * 0.95;
	} else if (x > width){
		x = width;
		xv = -xv * 0.95;
	}
	if(y > height){
		y = height;
		yv = -yv * 0.95;
	}
}

void Ball::draw(const Cairo::RefPtr<Cairo::Context>& cr){
	cr->arc(x, y, r, 0, 6.29);
	cr->fill_preserve();
	cr->stroke();
}

void Ball::collide(AnimObject *other){
	float xd = other->x - x;
	float yd = other->y - y;
	float td = fabs(xd) + fabs(yd);
	xd = (xd / td);
	yd = (yd / td);
	float txv = fabs(other->xv) + fabs(xv);
	float tyv = fabs(other->yv) + fabs(yv);
	float tv = txv * fabs(xd) + tyv * fabs(yd);
	float share = (other->mover)? 0.5:1.0;
	xv -= tv * xd * share;
	yv -= tv * yd * share;
	xv *= 0.95;
	yv *= 0.95;
}
